"use strict";
/*
The MIT License (MIT)

Copyright (c) 2014-2017 Bryan Hughes <bryan@nebri.us>

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/
Object.defineProperty(exports, "__esModule", { value: true });
const raspi_peripheral_1 = require("raspi-peripheral");
const pigpio_1 = require("pigpio");
const raspi_board_1 = require("raspi-board");
exports.LOW = 0;
exports.HIGH = 1;
exports.PULL_NONE = pigpio_1.Gpio.PUD_OFF;
exports.PULL_DOWN = pigpio_1.Gpio.PUD_DOWN;
exports.PULL_UP = pigpio_1.Gpio.PUD_UP;
function parseConfig(config) {
    let pin;
    let pullResistor;
    if (typeof config === 'number' || typeof config === 'string') {
        pin = config;
        pullResistor = exports.PULL_NONE;
    }
    else if (typeof config === 'object') {
        pin = config.pin;
        pullResistor = config.pullResistor || exports.PULL_NONE;
        if ([exports.PULL_NONE, exports.PULL_DOWN, exports.PULL_UP].indexOf(pullResistor) === -1) {
            throw new Error('Invalid pull resistor option ' + pullResistor);
        }
    }
    else {
        throw new Error('Invalid pin or configuration');
    }
    return {
        pin,
        pullResistor
    };
}
function getPin(alias, pin) {
    const gpioPin = raspi_board_1.getGpioNumber(pin);
    if (gpioPin === null) {
        throw new Error(`Internal error: ${alias} was parsed as a valid pin, but couldn't be resolved to a GPIO pin`);
    }
    return gpioPin;
}
class DigitalOutput extends raspi_peripheral_1.Peripheral {
    get value() {
        return this._currentValue;
    }
    constructor(config) {
        const parsedConfig = parseConfig(config);
        super(parsedConfig.pin);
        this._output = new pigpio_1.Gpio(getPin(parsedConfig.pin, this.pins[0]), {
            mode: pigpio_1.Gpio.OUTPUT,
            pullUpDown: parsedConfig.pullResistor
        });
        this._currentValue = this._output.digitalRead();
    }
    write(value) {
        if (!this.alive) {
            throw new Error('Attempted to write to a destroyed peripheral');
        }
        if ([exports.LOW, exports.HIGH].indexOf(value) === -1) {
            throw new Error('Invalid write value ' + value);
        }
        this._currentValue = value;
        this._output.digitalWrite(this.value);
        this.emit('change', this.value);
    }
}
exports.DigitalOutput = DigitalOutput;
class DigitalInput extends raspi_peripheral_1.Peripheral {
    get value() {
        return this._currentValue;
    }
    constructor(config) {
        const parsedConfig = parseConfig(config);
        super(parsedConfig.pin);
        this._input = new pigpio_1.Gpio(getPin(parsedConfig.pin, this.pins[0]), {
            mode: pigpio_1.Gpio.INPUT,
            pullUpDown: parsedConfig.pullResistor
        });
        this._input.enableInterrupt(pigpio_1.Gpio.EITHER_EDGE);
        this._input.on('interrupt', (level) => setTimeout(() => {
            this._currentValue = level;
            this.emit('change', this.value);
        }));
        this._currentValue = this._input.digitalRead();
    }
    destroy() {
        this._input.disableInterrupt();
        super.destroy();
    }
    read() {
        if (!this.alive) {
            throw new Error('Attempted to read from a destroyed peripheral');
        }
        this._currentValue = this._input.digitalRead();
        return this.value;
    }
}
exports.DigitalInput = DigitalInput;
exports.module = {
    PULL_DOWN: exports.PULL_DOWN,
    PULL_UP: exports.PULL_UP,
    PULL_NONE: exports.PULL_NONE,
    createDigitalInput(config) {
        return new DigitalInput(config);
    },
    createDigitalOutput(config) {
        return new DigitalOutput(config);
    }
};
//# sourceMappingURL=index.js.map